LOVE_LIGHTMAP_SHADER = love.graphics.newShader("shader/shadow.glsl")
LOVE_LIGHTMAP_LAST_BUFFER = nil
LOVE_LIGHTMAP_SHADER:send("screenSize", {love.graphics.getWidth(), love.graphics.getHeight()})

love.lightmap = {}

love.lightmap.newWorld = function()
	local o = {}
	o.canvas = love.graphics.newCanvas()
	o.lights = {}
	o.ambientColor = {0, 0, 0}
	o.shadowMap = nil
	-- set ambient color
	o.setAmbientColor = function(red, green, blue)
		o.ambientColor = {red, green, blue}
	end
	-- set shadow map
	o.setShadowCanvas = function(canvas)
		o.shadowMap = canvas
	end
	-- add new light
	o.newLight = function(x, y, red, green, blue, range)
		return love.lightmap.newLight(o, x, y, red, green, blue, range)
	end
	-- update lightmap
	o.update = function(dt)
		LOVE_LIGHTMAP_LAST_BUFFER = love.graphics.getCanvas()
		love.graphics.setCanvas(o.canvas)
		love.graphics.setBlendMode("alpha")
		love.graphics.setColor(o.ambientColor[1], o.ambientColor[2], o.ambientColor[3])
		love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())
		love.graphics.setBlendMode("additive")
		love.graphics.setShader(LOVE_LIGHTMAP_SHADER)
		for i = 1, #o.lights do
			LOVE_LIGHTMAP_SHADER:send("lightPosition", {o.lights[i].x, 600 - o.lights[i].y})
			LOVE_LIGHTMAP_SHADER:send("lightColorRange", {o.lights[i].color[1] / 255, o.lights[i].color[2] / 255, o.lights[i].color[3] / 255, o.lights[i].range})
			--LOVE_LIGHTMAP_SHADER:send("lightAmbient", {o.ambientColor[1] / 255, o.ambientColor[2] / 255, o.ambientColor[3] / 255})
			love.graphics.draw(o.shadowMap)
		end
		love.graphics.setCanvas(LOVE_LIGHTMAP_LAST_BUFFER)
		love.graphics.setShader()
		love.graphics.setColor(255, 255, 255)
	end
	-- draw lightmap
	o.draw = function(ox, oy)
		love.graphics.setShader()
		love.graphics.setColor(255, 255, 255)
		love.graphics.setBlendMode("multiplicative")
		love.graphics.draw(o.canvas, ox, oy)
		love.graphics.setBlendMode("alpha")
	end

	return o
end

love.lightmap.newLight = function(p, x, y, red, green, blue, range)
	local o = {}
	p.lights[#p.lights + 1] = o
	o.x = x or 0
	o.y = y or 0
	o.color = {(red or 255), (green or 255), (blue or 255)}
	o.range = range or 200
	-- get x position
	o.getX = function()
		return o.x
	end
	-- get y position
	o.getY = function()
		return o.y
	end
	-- set position
	o.setPosition = function(x, y)
		o.x = x
		o.y = y
	end
	-- set color
	o.setColor = function(red, green, blue)
		o.color = {red, green, blue}
	end
	-- set range
	o.setRange = function(range)
		o.range = range
	end

	return o
end