require "postshader"
require "lightmap"

function love.load()
	paintCanvas = love.graphics.newCanvas()
	shadowCanvas = love.graphics.newCanvas()
	paintImage = love.graphics.newImage("shadow.png")
	floorImage = love.graphics.newImage("floor.png")
	floorImage:setWrap("repeat", "repeat")
	lightImage = love.graphics.newImage("light.png")
	playerImage = love.graphics.newImage("player.png")
	rotationImage = love.graphics.newImage("rotation.png")
	quad = love.graphics.newQuad(0, 0, love.window.getWidth(), love.window.getHeight(), 32, 32)

	love.graphics.setCanvas(paintCanvas)
	love.graphics.draw(paintImage)
	love.graphics.setCanvas()

	playerX = 256
	playerY = 320
	lightTime = 0

	-- create physic world
	physicWorld = love.physics.newWorld(0, 0, true)
	myPhysicBlock1 = {}
	myPhysicBlock1.body = love.physics.newBody(physicWorld, 320, 320, "dynamic")
	myPhysicBlock1.shape = love.physics.newRectangleShape(0, 0, 64, 32)
	myPhysicBlock1.fixture = love.physics.newFixture(myPhysicBlock1.body, myPhysicBlock1.shape)
	
	-- create light world
	lightWorld = love.lightmap.newWorld()
	lightWorld.setAmbientColor(31, 31, 31)
	lightWorld.setShadowCanvas(shadowCanvas)

	-- create lights
	myLight1 = lightWorld.newLight(playerX, playerY, 255, 191, 127, 500)
	myLight2 = lightWorld.newLight(512, 256, 63, 255, 127, 500)
	myLight3 = lightWorld.newLight(64, 64, 127, 191, 255, 500)
end

function love.update(dt)
	-- show fps
	love.window.setTitle("FPS:" .. love.timer.getFPS())

	-- move player
	if love.keyboard.isDown("a") then
		playerX = playerX - dt * 100
	elseif love.keyboard.isDown("d") then
		playerX = playerX + dt * 100
	end

	if love.keyboard.isDown("w") then
		playerY = playerY - dt * 100
	elseif love.keyboard.isDown("s") then
		playerY = playerY + dt * 100
	end

	myLight1.setPosition(playerX, playerY)

	-- update physics block
	myPhysicBlock1.body:applyTorque(-75)

	-- draw into paint buffer
	love.graphics.setBlendMode("replace")

	if love.mouse.isDown("l") then
		love.graphics.setCanvas(paintCanvas)
		love.graphics.setColor(95, 127, 191)
		love.graphics.circle("fill", love.mouse.getX(), love.mouse.getY(), 4)
		love.graphics.setCanvas()
	elseif love.mouse.isDown("r") then
		love.graphics.setCanvas(paintCanvas)
		love.graphics.setColor(0, 0, 0, 0)
		love.graphics.circle("fill", love.mouse.getX(), love.mouse.getY(), 4)
		love.graphics.setCanvas()
	elseif love.mouse.isDown("m") then
		-- move light 2
		myLight2.setPosition(love.mouse.getX(), love.mouse.getY())
	end

	-- update shadow buffer
	love.graphics.setBlendMode("replace")
	love.graphics.setCanvas(shadowCanvas)
	love.graphics.setColor(255, 255, 255)

	-- draw paint buffer
	love.graphics.draw(paintCanvas)

	-- draw physic block
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(63, 63, 63)
	love.graphics.polygon("fill", myPhysicBlock1.body:getWorldPoints(myPhysicBlock1.shape:getPoints()))
	love.graphics.draw(rotationImage, myPhysicBlock1.body:getX(), myPhysicBlock1.body:getY(), myPhysicBlock1.body:getAngle(), 1, 1, 7, 6)

	love.graphics.setCanvas()

	-- move light 3
	lightTime = lightTime + dt
	myLight3.setPosition(math.sin(lightTime) * 32 + 64, math.cos(lightTime) * 32 + 64)

	-- update physics
	physicWorld:update(dt)

	-- update lightmap
	lightWorld.update(dt)
end

function love.draw()
	-- set shader buffer
	love.postshader.setBuffer("render")

	-- draw scene
	love.graphics.setColor(255, 255, 255)
	love.graphics.setBlendMode("alpha")
	love.graphics.draw(floorImage, quad)
	love.graphics.draw(shadowCanvas)
	love.graphics.draw(playerImage, playerX, playerY)
	love.graphics.draw(lightImage, myLight2.getX(), myLight2.getY())
	love.graphics.draw(lightImage, myLight3.getX(), myLight3.getY())

	-- draw lightmap
	lightWorld.draw()

	-- draw help
	love.graphics.setColor(0, 0, 0, 191)
	love.graphics.rectangle("fill", 8, 8, 210, 80)
	love.graphics.setColor(255, 255, 255)
	love.graphics.print("WASD: Move player", 16, 16)
	love.graphics.print("Mouse left click: Draw wall", 16, 32)
	love.graphics.print("Mouse right click: Erase wall", 16, 48)
	love.graphics.print("Mouse middle click: Move light", 16, 64)

	-- draw shader
	love.postshader.draw()
end

function love.mousepressed(x, y, button)

end